CHROMEDRIVER_CHROMIUM_URL = 'https://chromedriver.chromium.org'
CHROMEDRIVER_STORAGE_URL = 'https://chromedriver.storage.googleapis.com'
LATEST_RELEASE_URL = CHROMEDRIVER_STORAGE_URL + 'LATEST_RELEASE'

FILE_NAME_CHROMEDRIVER = 'chromedriver'
FILE_NAME_CHROMEDRIVER_EXE = 'chromedriver.exe'
FILE_NAME_CHROMEDRIVER_WIN_ZIP = 'chromedriver_win32.zip'
FILE_NAME_CHROMEDRIVER_LINUX_ZIP = 'chromedriver_linux64.zip'
FILE_NAME_CHROMEDRIVER_MAC_ZIP = 'chromedriver_mac64.zip'

DIR_DOWNLOAD = 'chromedriver'

UL_RELEASES_SELECTOR = '#sites-canvas-main-content > table > tbody > tr > td > div > div:nth-child(5) > ul'

LATEST_STABLE_RELEASE_STR = 'Latest stable release'
LATEST_BETA_RELEASE_STR = 'Latest beta release'
